/**
 * # Bodyguards
 *
 * Bodyguards are guards that operate on the body (*request.query,
 * request.body*). Bodyguards come in two forms:
 * - Guards
 *	- Guards block incoming requests, e.g. a basic user
 *	who tries to delete an admin
 * - Filters
 *	- Filters remove sensitive fields from response bodies
 *
 * ## Decorators
 *
 * Decorators are used to set read/write privelages on your model. There
 * are several decorators already available for you:
 * - Read
 *  - CanRead(who?)
 *  - AnyoneCanRead()
 *  - OnlySelfCanRead()
 *  - OnlyAdminCanRead()
 * - Write
 *  - CanWrite(who?)
 *  - AnyoneCanWrite()
 *  - OnlySelfCanWrite()
 *  - OnlyAdminCanWrite()
 * - Search
 *  - CanSearch(who?)
 *
 * Decorators which take a *who* parameter default to anyone but also
 * can take a UserRole or BodyguardOwner enum
 *
 * **NOTE: If a member does not have a CanRead or CanWrite key, it will
 * not be able to be read or written!**
 *
 * ### BodyguardKey
 *
 * There is one additional decorator: `BodyguardKey`. This key will be
 * presented to the guards/filters to authorize the request. For example, the
 * `userId` of a User would be the BodyguardKey, or the `fromUserId` and
 * `toUserId` would be the BodyguardKeys on a Chat model. When the
 * bodyguard runs, it will compare the authenticated user with the
 * BodyguardKey of the resource.
 *
 * ## How to Use?
 *
 * Guards, Endpoints, and Responders contain bodyguards underneath. If you
 * are not writing custom guards or responders, you will only need to set the
 * decorators on your model.
 *
 * If you are writing custom guards or responders, please see the respective
 * documentation to learn more about integrating bodyguard functions
 *
 * ## How to Set Privelages?
 *
 * You will need to add the appropriate bodyguards on your model:
 *
 * ```typescript
 * import { ExampleUser } from 'pointyapi/models';
 * import {
 * 	CanRead,
 * 	AnyoneCanRead,
 * 	OnlySelfCanRead,
 * 	OnlyAdminCanRead,
 * 	CanWrite,
 * 	AnyoneCanWrite,
 * 	OnlySelfCanWrite,
 * 	OnlyAdminCanWrite,
 * 	CanSearch
 * } from 'pointyapi/bodyguards';
 * import { UserRole, UserStatus, BodyguardOwner } from 'pointyapi/enums';
 *
 * export class User extends BaseUser {
 *		// ID
 *		@BodyguardKey() // Determine who can read/write from here
 *		@AnyoneCanRead()
 *		public id: number = undefined;
 *
 *		// Time created
 *		@AnyoneCanRead()
 *		public timeCreated: Date = undefined;
 *
 *		// Time last updated
 *		@AnyoneCanRead()
 *		public timeUpdated: Date = undefined;
 *
 *		// Username
 *		@AnyoneCanRead()
 *		@OnlySelfCanWrite()
 *		@CanSearch()
 *		public username: string = undefined;
 *
 *		// First name
 *		@AnyoneCanRead()
 *		@OnlySelfCanWrite()
 *		@CanSearch()
 *		public fname: string = undefined;
 *
 *		// User Role
 *		@OnlySelfCanRead()
 *		@OnlyAdminCanWrite()
 *		public role: UserRole = undefined;
 * }
 *
 * export class Chat extends BaseModel() {
 *		// ID
 *		@OnlySelfCanRead()
 *		public id: number = undefined;
 *
 *		// Time created
 *		@OnlySelfCanRead()
 *		public timeCreated: Date = undefined;
 *
 *		// Time last updated
 *		@OnlySelfCanRead()
 *		public timeUpdated: Date = undefined;
 *
 * 		// From User
 * 		@BodyguardKey() // Determine who can read/write from here
 * 		@OnlySelfCanRead()
 * 		public fromUserId: number = undefined;
 *
 * 		// To User
 * 		@BodyguardKey() // Determine who can read/write from here
 * 		@OnlySelfCanRead()
 * 		public toUserId: number = undefined;
 * }
 *
 * ```
 *
 */
/**
 * # Bodyguard Decorators
 */
import 'reflect-metadata';
import { BodyguardOwner, UserRole } from './enums';
/**
 * ## Class Decorators
 */
/**
 * Get if all members are readable
 * NOTE:	This function only checks if the CanReadAll() decorator is present.
 * 			Individual fields may have fine-grain access rights, which this
 * 			function will ignore. Use getCanRead() instead for authorization.
 * @param target Object to test
 */
export declare function getCanReadAll(target: any): string;
/**
 * Get if all members are writableF
 * NOTE:	This function only checks if the CanWriteAll() decorator is present.
 * 			Individual fields may have fine-grain access rights, which this
 * 			function will ignore. Use getCanWrite() instead for authorization.
 * @param target Object to test
 */
export declare function getCanWriteAll(target: any): string;
/**
 * All class members are readable
 * @param who Who can read the fields. Default is anyone
 */
export declare function CanReadAll(who?: BodyguardOwner | UserRole): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * All class members are readable
 * @param who Who can read the fields. Default is anyone
 */
export declare function CanWriteAll(who?: BodyguardOwner | UserRole): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * ## Member Decorators
 */
/**
 * Check if the key is a bodyguard key
 * @param target Object to test
 * @param propertyKey Key to check
 */
export declare function isBodyguardKey(target: any, propertyKey: string): boolean;
/**
 * Get the read privilege of the key
 * @param target Object to test
 * @param propertyKey Key to check
 */
export declare function getCanRead(target: any, propertyKey: string): string;
/**
 * Get the write privilege of the key
 * @param target Object to test
 * @param propertyKey Key to check
 */
export declare function getCanWrite(target: any, propertyKey: string): string;
/**
 * Get the search privilege of the key
 * @param target Object to test
 * @param propertyKey Key to check
 */
export declare function getCanSearch(target: any, propertyKey: string): string;
/**
 * Get the search relations of the key
 * @param target Object to test
 * @param propertyKey Key to check
 */
export declare function getCanSearchRelation(target: any, propertyKey: string): any;
/**
 * Get the read relations of the key
 * @param target Object to test
 * @param propertyKey Key to check
 */
export declare function getCanReadRelation(target: any, propertyKey: string): any;
/**
 * Bodyguard Key: This key represents ownership of the object
 * 	This might be `id` for a User, or `from` and `to` for a Chat
 * 	This key will be presented to guards and filters to determine
 * 	if the user isSelf or owns the object
 */
export declare function BodyguardKey(): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Anyone can read the field
 */
export declare function AnyoneCanRead(): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Anyone can write the field
 */
export declare function AnyoneCanWrite(): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Only owner can read the field
 */
export declare function OnlySelfCanRead(): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Only owner can write the field
 */
export declare function OnlySelfCanWrite(): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Only admin can read the field
 */
export declare function OnlyAdminCanRead(): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Only admin can write the field
 */
export declare function OnlyAdminCanWrite(): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Sets the read privilege of the field
 * @param who Who can read the field
 */
export declare function CanRead(who: BodyguardOwner | UserRole): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Sets the write privilege of the field
 * @param who Who can read the field
 */
export declare function CanWrite(who: BodyguardOwner | UserRole): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Sets who can search the field
 * @param who Who can search the field. Default is anyone
 */
export declare function CanSearch(who?: BodyguardOwner | UserRole): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Sets who can search relations in the field
 * @param who Object with who (string) and field (array)
 *
 * {
 * 		who: BodyguardOwner.Self,
 * 		fields: [ 'username', 'fname', 'lname' ]
 * }
 */
export declare function CanSearchRelation(params: object): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
/**
 * Sets who can read the relation
 * @param who Who can read the relation. Default is anyone
 */
export declare function CanReadRelation(who?: BodyguardOwner | UserRole): {
    (target: Function): void;
    (target: Object, propertyKey: string | symbol): void;
};
export { compareNestedBodyguards } from './bodyguard/compare-nested';
export { compareIdToUser } from './bodyguard/compare-id-to-user';
export { getBodyguardKeys } from './bodyguard/get-bodyguard-keys';
export { getSearchableFields } from './bodyguard/get-searchable-fields';
export { getSearchableRelations } from './bodyguard/get-searchable-relations';
export { getReadableFields } from './bodyguard/get-readable-fields';
export { getReadableRelations } from './bodyguard/get-readable-relations';
export { readFilter } from './bodyguard/read-filter';
export { writeFilter } from './bodyguard/write-filter';
