"use strict";
/**
 * # Bodyguards
 *
 * Bodyguards are guards that operate on the body (*request.query,
 * request.body*). Bodyguards come in two forms:
 * - Guards
 *	- Guards block incoming requests, e.g. a basic user
 *	who tries to delete an admin
 * - Filters
 *	- Filters remove sensitive fields from response bodies
 *
 * ## Decorators
 *
 * Decorators are used to set read/write privelages on your model. There
 * are several decorators already available for you:
 * - Read
 *  - CanRead(who?)
 *  - AnyoneCanRead()
 *  - OnlySelfCanRead()
 *  - OnlyAdminCanRead()
 * - Write
 *  - CanWrite(who?)
 *  - AnyoneCanWrite()
 *  - OnlySelfCanWrite()
 *  - OnlyAdminCanWrite()
 * - Search
 *  - CanSearch(who?)
 *
 * Decorators which take a *who* parameter default to anyone but also
 * can take a UserRole or BodyguardOwner enum
 *
 * **NOTE: If a member does not have a CanRead or CanWrite key, it will
 * not be able to be read or written!**
 *
 * ### BodyguardKey
 *
 * There is one additional decorator: `BodyguardKey`. This key will be
 * presented to the guards/filters to authorize the request. For example, the
 * `userId` of a User would be the BodyguardKey, or the `fromUserId` and
 * `toUserId` would be the BodyguardKeys on a Chat model. When the
 * bodyguard runs, it will compare the authenticated user with the
 * BodyguardKey of the resource.
 *
 * ## How to Use?
 *
 * Guards, Endpoints, and Responders contain bodyguards underneath. If you
 * are not writing custom guards or responders, you will only need to set the
 * decorators on your model.
 *
 * If you are writing custom guards or responders, please see the respective
 * documentation to learn more about integrating bodyguard functions
 *
 * ## How to Set Privelages?
 *
 * You will need to add the appropriate bodyguards on your model:
 *
 * ```typescript
 * import { ExampleUser } from 'pointyapi/models';
 * import {
 * 	CanRead,
 * 	AnyoneCanRead,
 * 	OnlySelfCanRead,
 * 	OnlyAdminCanRead,
 * 	CanWrite,
 * 	AnyoneCanWrite,
 * 	OnlySelfCanWrite,
 * 	OnlyAdminCanWrite,
 * 	CanSearch
 * } from 'pointyapi/bodyguards';
 * import { UserRole, UserStatus, BodyguardOwner } from 'pointyapi/enums';
 *
 * export class User extends BaseUser {
 *		// ID
 *		@BodyguardKey() // Determine who can read/write from here
 *		@AnyoneCanRead()
 *		public id: number = undefined;
 *
 *		// Time created
 *		@AnyoneCanRead()
 *		public timeCreated: Date = undefined;
 *
 *		// Time last updated
 *		@AnyoneCanRead()
 *		public timeUpdated: Date = undefined;
 *
 *		// Username
 *		@AnyoneCanRead()
 *		@OnlySelfCanWrite()
 *		@CanSearch()
 *		public username: string = undefined;
 *
 *		// First name
 *		@AnyoneCanRead()
 *		@OnlySelfCanWrite()
 *		@CanSearch()
 *		public fname: string = undefined;
 *
 *		// User Role
 *		@OnlySelfCanRead()
 *		@OnlyAdminCanWrite()
 *		public role: UserRole = undefined;
 * }
 *
 * export class Chat extends BaseModel() {
 *		// ID
 *		@OnlySelfCanRead()
 *		public id: number = undefined;
 *
 *		// Time created
 *		@OnlySelfCanRead()
 *		public timeCreated: Date = undefined;
 *
 *		// Time last updated
 *		@OnlySelfCanRead()
 *		public timeUpdated: Date = undefined;
 *
 * 		// From User
 * 		@BodyguardKey() // Determine who can read/write from here
 * 		@OnlySelfCanRead()
 * 		public fromUserId: number = undefined;
 *
 * 		// To User
 * 		@BodyguardKey() // Determine who can read/write from here
 * 		@OnlySelfCanRead()
 * 		public toUserId: number = undefined;
 * }
 *
 * ```
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * # Bodyguard Decorators
 */
require("reflect-metadata");
var enums_1 = require("./enums");
var CanReadAllSymbol = Symbol('CanReadAll');
var CanWriteAllSymbol = Symbol('CanWriteAll');
var CanReadSymbol = Symbol('CanRead');
var CanWriteSymbol = Symbol('CanWrite');
var BodyguardKeySymbol = Symbol('BodyguardKey');
var CanSearchSymbol = Symbol('CanSearch');
var CanSearchRelationSymbol = Symbol('CanSearchRelation');
var CanReadRelationSymbol = Symbol('CanReadRelationSymbol');
/**
 * ## Class Decorators
 */
/**
 * Get if all members are readable
 * NOTE:	This function only checks if the CanReadAll() decorator is present.
 * 			Individual fields may have fine-grain access rights, which this
 * 			function will ignore. Use getCanRead() instead for authorization.
 * @param target Object to test
 */
function getCanReadAll(target) {
    return Reflect.getMetadata(CanReadAllSymbol, target.constructor);
}
exports.getCanReadAll = getCanReadAll;
/**
 * Get if all members are writableF
 * NOTE:	This function only checks if the CanWriteAll() decorator is present.
 * 			Individual fields may have fine-grain access rights, which this
 * 			function will ignore. Use getCanWrite() instead for authorization.
 * @param target Object to test
 */
function getCanWriteAll(target) {
    return Reflect.getMetadata(CanWriteAllSymbol, target.constructor);
}
exports.getCanWriteAll = getCanWriteAll;
/**
 * All class members are readable
 * @param who Who can read the fields. Default is anyone
 */
function CanReadAll(who) {
    if (who === void 0) { who = enums_1.BodyguardOwner.Anyone; }
    return Reflect.metadata(CanReadAllSymbol, who);
}
exports.CanReadAll = CanReadAll;
/**
 * All class members are readable
 * @param who Who can read the fields. Default is anyone
 */
function CanWriteAll(who) {
    if (who === void 0) { who = enums_1.BodyguardOwner.Anyone; }
    return Reflect.metadata(CanWriteAllSymbol, who);
}
exports.CanWriteAll = CanWriteAll;
/**
 * ## Member Decorators
 */
/**
 * Check if the key is a bodyguard key
 * @param target Object to test
 * @param propertyKey Key to check
 */
function isBodyguardKey(target, propertyKey) {
    return Reflect.getMetadata(BodyguardKeySymbol, target, propertyKey);
}
exports.isBodyguardKey = isBodyguardKey;
/**
 * Get the read privilege of the key
 * @param target Object to test
 * @param propertyKey Key to check
 */
function getCanRead(target, propertyKey) {
    return (Reflect.getMetadata(CanReadSymbol, target, propertyKey) ||
        getCanReadAll(target));
}
exports.getCanRead = getCanRead;
/**
 * Get the write privilege of the key
 * @param target Object to test
 * @param propertyKey Key to check
 */
function getCanWrite(target, propertyKey) {
    return (Reflect.getMetadata(CanWriteSymbol, target, propertyKey) ||
        getCanWriteAll(target));
}
exports.getCanWrite = getCanWrite;
/**
 * Get the search privilege of the key
 * @param target Object to test
 * @param propertyKey Key to check
 */
function getCanSearch(target, propertyKey) {
    return Reflect.getMetadata(CanSearchSymbol, target, propertyKey);
}
exports.getCanSearch = getCanSearch;
/**
 * Get the search relations of the key
 * @param target Object to test
 * @param propertyKey Key to check
 */
function getCanSearchRelation(target, propertyKey) {
    return Reflect.getMetadata(CanSearchRelationSymbol, target, propertyKey);
}
exports.getCanSearchRelation = getCanSearchRelation;
/**
 * Get the read relations of the key
 * @param target Object to test
 * @param propertyKey Key to check
 */
function getCanReadRelation(target, propertyKey) {
    return Reflect.getMetadata(CanReadRelationSymbol, target, propertyKey);
}
exports.getCanReadRelation = getCanReadRelation;
/**
 * Bodyguard Key: This key represents ownership of the object
 * 	This might be `id` for a User, or `from` and `to` for a Chat
 * 	This key will be presented to guards and filters to determine
 * 	if the user isSelf or owns the object
 */
function BodyguardKey() {
    return Reflect.metadata(BodyguardKeySymbol, true);
}
exports.BodyguardKey = BodyguardKey;
/**
 * Anyone can read the field
 */
function AnyoneCanRead() {
    return Reflect.metadata(CanReadSymbol, enums_1.BodyguardOwner.Anyone);
}
exports.AnyoneCanRead = AnyoneCanRead;
/**
 * Anyone can write the field
 */
function AnyoneCanWrite() {
    return Reflect.metadata(CanWriteSymbol, enums_1.BodyguardOwner.Anyone);
}
exports.AnyoneCanWrite = AnyoneCanWrite;
/**
 * Only owner can read the field
 */
function OnlySelfCanRead() {
    return Reflect.metadata(CanReadSymbol, enums_1.BodyguardOwner.Self);
}
exports.OnlySelfCanRead = OnlySelfCanRead;
/**
 * Only owner can write the field
 */
function OnlySelfCanWrite() {
    return Reflect.metadata(CanWriteSymbol, enums_1.BodyguardOwner.Self);
}
exports.OnlySelfCanWrite = OnlySelfCanWrite;
/**
 * Only admin can read the field
 */
function OnlyAdminCanRead() {
    return Reflect.metadata(CanReadSymbol, enums_1.BodyguardOwner.Admin);
}
exports.OnlyAdminCanRead = OnlyAdminCanRead;
/**
 * Only admin can write the field
 */
function OnlyAdminCanWrite() {
    return Reflect.metadata(CanWriteSymbol, enums_1.BodyguardOwner.Admin);
}
exports.OnlyAdminCanWrite = OnlyAdminCanWrite;
/**
 * Sets the read privilege of the field
 * @param who Who can read the field
 */
function CanRead(who) {
    return Reflect.metadata(CanReadSymbol, who);
}
exports.CanRead = CanRead;
/**
 * Sets the write privilege of the field
 * @param who Who can read the field
 */
function CanWrite(who) {
    return Reflect.metadata(CanWriteSymbol, who);
}
exports.CanWrite = CanWrite;
/**
 * Sets who can search the field
 * @param who Who can search the field. Default is anyone
 */
function CanSearch(who) {
    if (who === void 0) { who = enums_1.BodyguardOwner.Anyone; }
    return Reflect.metadata(CanSearchSymbol, who);
}
exports.CanSearch = CanSearch;
/**
 * Sets who can search relations in the field
 * @param who Object with who (string) and field (array)
 *
 * {
 * 		who: BodyguardOwner.Self,
 * 		fields: [ 'username', 'fname', 'lname' ]
 * }
 */
function CanSearchRelation(params) {
    return Reflect.metadata(CanSearchRelationSymbol, params);
}
exports.CanSearchRelation = CanSearchRelation;
/**
 * Sets who can read the relation
 * @param who Who can read the relation. Default is anyone
 */
function CanReadRelation(who) {
    if (who === void 0) { who = enums_1.BodyguardOwner.Anyone; }
    return Reflect.metadata(CanReadRelationSymbol, who);
}
exports.CanReadRelation = CanReadRelation;
var compare_nested_1 = require("./bodyguard/compare-nested");
exports.compareNestedBodyguards = compare_nested_1.compareNestedBodyguards;
var compare_id_to_user_1 = require("./bodyguard/compare-id-to-user");
exports.compareIdToUser = compare_id_to_user_1.compareIdToUser;
var get_bodyguard_keys_1 = require("./bodyguard/get-bodyguard-keys");
exports.getBodyguardKeys = get_bodyguard_keys_1.getBodyguardKeys;
var get_searchable_fields_1 = require("./bodyguard/get-searchable-fields");
exports.getSearchableFields = get_searchable_fields_1.getSearchableFields;
var get_searchable_relations_1 = require("./bodyguard/get-searchable-relations");
exports.getSearchableRelations = get_searchable_relations_1.getSearchableRelations;
var get_readable_fields_1 = require("./bodyguard/get-readable-fields");
exports.getReadableFields = get_readable_fields_1.getReadableFields;
var get_readable_relations_1 = require("./bodyguard/get-readable-relations");
exports.getReadableRelations = get_readable_relations_1.getReadableRelations;
var read_filter_1 = require("./bodyguard/read-filter");
exports.readFilter = read_filter_1.readFilter;
var write_filter_1 = require("./bodyguard/write-filter");
exports.writeFilter = write_filter_1.writeFilter;
//# sourceMappingURL=bodyguard.js.map