"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bodyguard_1 = require("../bodyguard");
var utils_1 = require("../utils");
var enums_1 = require("../enums");
/**
 * Filter an incoming request body to ensure it doesn't allow sensitive fields
 * @param obj Object or array to check (recursive).
 * 	This can be the direct obj set of a typeorm request
 * @param user User object to check against
 * @param objType Type of object
 * @param userType Type of user
 * @param isSelfResult If the usr owns the object
 * @return Returns true on success, or a string of the member name which failed.
 */
function writeFilter(obj, user, objType, userType, isSelfResult) {
    if (obj instanceof Array) {
        // Recurse array of objects
        for (var i = 0; i < obj.length; i++) {
            var result = writeFilter(obj[i], user, objType, userType, isSelfResult);
            if (result !== true) {
                return "[#" + i + "]" + result;
            }
        }
    }
    else if (obj instanceof Object) {
        // Check if user owns the object
        if (isSelfResult === undefined) {
            isSelfResult = utils_1.isSelf(obj, user, objType, userType);
        }
        // Check if user is admin
        var isAdminResult = utils_1.isAdmin(user);
        // Loop through object members
        for (var member in obj) {
            if (!(obj[member] instanceof Function)) {
                var canWrite = bodyguard_1.getCanWrite(new objType(), member);
                if (canWrite === undefined) {
                    // Cannot write PointyAPI special keys
                    if (member.indexOf('__') !== 0) {
                        return member;
                    }
                }
                else if (canWrite !== enums_1.BodyguardOwner.Anyone &&
                    ((canWrite === enums_1.BodyguardOwner.Self && !isSelfResult) ||
                        (canWrite === enums_1.BodyguardOwner.Admin && !isAdminResult))) {
                    // No permission to write this field
                    return member;
                }
            }
        }
    }
    return true;
}
exports.writeFilter = writeFilter;
//# sourceMappingURL=write-filter.js.map