"use strict";
/**
 * # Endpoints
 *
 * Endpoints are where the action of a route actually takes place,
 * for example posting a resource or logging in. The following
 * endpoints come premade:
 * - CRUD
 *  - postEndpoint
 *  - getEndpoint
 *  - patchEndpoint
 *  - deleteEndpoint
 * - Auth
 *  - loginEndpoint
 *  - logoutEndpoint
 *
 * ## Using endpoints
 *
 * Endpoints go in your route, after any guards. You'll have to
 * set a model so the guards and endpoints know what you're working with.
 *
 * ```typescript
 * import { setModel } from 'pointyapi/set-model';
 * import { ExampleUser } from 'pointyapi/models';
 *
 * // Guards
 * import { onlySelf } from 'pointyapi/guards';
 *
 * // Filters
 * import { postFilter, getFilter, patchFilter } from 'pointyapi/filters';
 *
 * // Endpoints
 * import {
 * 		postEndpoint,
 * 		getEndpoint,
 * 		patchEndpoint,
 * 		deleteEndpoint
 * } from 'pointyapi/endpoints';
 *
 * // Create router
 * const router: Router = Router();
 *
 * // Set model
 * async function loader(request, response, next) {
 * 		if (await setModel(request, ExampleUser)) {
 * 			next();
 * 		}
 * }
 *
 * // Set routes
 * router.post('/', loader, postFilter, postEndpoint);
 * router.get('/', loader, getFilter, getEndpoint);
 * router.patch(`/:id`, loader, onlySelf, patchFilter, patchEndpoint);
 * router.delete(`/:id`, loader, onlySelf, deleteEndpoint);
 *
 * // Export router
 * export const userRouter: Router = router;
 * ```
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Endpoints
 */
var delete_endpoint_1 = require("./endpoints/delete-endpoint");
exports.deleteEndpoint = delete_endpoint_1.deleteEndpoint;
var get_endpoint_1 = require("./endpoints/get-endpoint");
exports.getEndpoint = get_endpoint_1.getEndpoint;
var post_endpoint_1 = require("./endpoints/post-endpoint");
exports.postEndpoint = post_endpoint_1.postEndpoint;
var patch_endpoint_1 = require("./endpoints/patch-endpoint");
exports.patchEndpoint = patch_endpoint_1.patchEndpoint;
var login_endpoint_1 = require("./endpoints/login-endpoint");
exports.loginEndpoint = login_endpoint_1.loginEndpoint;
var logout_endpoint_1 = require("./endpoints/logout-endpoint");
exports.logoutEndpoint = logout_endpoint_1.logoutEndpoint;
var refresh_token_endpoint_1 = require("./endpoints/refresh-token-endpoint");
exports.refreshTokenEndpoint = refresh_token_endpoint_1.refreshTokenEndpoint;
//# sourceMappingURL=endpoints.js.map