"use strict";
/**
 * # HttpClient
 *
 * HttpClient can be used to mock requests in your test-suite. There are
 * 4 functions: post(), get(), patch(), delete()
 *
 * **Before using, set HttpClient.url and HttpClient.port**
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * HttpClient
 */
var request = require("request");
var http_client_response_1 = require("./http-client-response");
var HttpClient = /** @class */ (function () {
    function HttpClient() {
        this.request = request;
        // Server URL
        this.url = 'http://localhost';
        // Server PORT
        this.port = process.env.PORT;
    }
    /**
     * Send a POST http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    HttpClient.prototype.post = function (path, data, bearer, expect, customOptions) {
        var _this = this;
        if (bearer === void 0) { bearer = false; }
        if (expect === void 0) { expect = [200, 201, 202, 204]; }
        var options = {
            method: 'POST',
            url: this.url + ":" + this.port + path,
            body: data,
            json: true
        };
        if (bearer) {
            options['auth'] = { bearer: "" + bearer };
        }
        if (this.headers) {
            options['headers'] = this.headers;
        }
        if (customOptions) {
            options = Object.assign(options, customOptions);
        }
        return new Promise(function (accept, reject) {
            _this.request(options, function (error, response, body) {
                if (error) {
                    reject(new http_client_response_1.HttpClientResponse(response, error));
                }
                else if (response) {
                    if (expect.includes(response.statusCode)) {
                        accept(new http_client_response_1.HttpClientResponse(response, body));
                    }
                    else {
                        reject(new http_client_response_1.HttpClientResponse(response, error));
                    }
                }
                else {
                    reject(new http_client_response_1.HttpClientResponse(response, 'No response'));
                }
            });
        });
    };
    /**
     * Send a GET http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    HttpClient.prototype.get = function (path, data, bearer, expect, customOptions) {
        var _this = this;
        if (data === void 0) { data = false; }
        if (bearer === void 0) { bearer = false; }
        if (expect === void 0) { expect = [200, 202]; }
        return new Promise(function (accept, reject) {
            var options = {
                method: 'GET',
                url: _this.url + ":" + _this.port + path,
                json: true,
                qs: data
            };
            if (bearer) {
                options['auth'] = { bearer: "" + bearer };
            }
            if (_this.headers) {
                options['headers'] = _this.headers;
            }
            if (customOptions) {
                options = Object.assign(options, customOptions);
            }
            _this.request(options, function (error, response, body) {
                if (error) {
                    reject(new http_client_response_1.HttpClientResponse(response, error));
                }
                else if (response) {
                    if (expect.includes(response.statusCode)) {
                        accept(new http_client_response_1.HttpClientResponse(response, body));
                    }
                    else {
                        reject(new http_client_response_1.HttpClientResponse(response, error));
                    }
                }
                else {
                    reject(new http_client_response_1.HttpClientResponse(response, 'No response'));
                }
            });
        });
    };
    /**
     * Send a PATCH http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    HttpClient.prototype.patch = function (path, data, bearer, expect, customOptions) {
        var _this = this;
        if (bearer === void 0) { bearer = false; }
        if (expect === void 0) { expect = [200, 201, 202, 204]; }
        var options = {
            method: 'PATCH',
            url: this.url + ":" + this.port + path,
            body: data,
            json: true
        };
        if (bearer) {
            options['auth'] = { bearer: "" + bearer };
        }
        if (this.headers) {
            options['headers'] = this.headers;
        }
        if (customOptions) {
            options = Object.assign(options, customOptions);
        }
        return new Promise(function (accept, reject) {
            _this.request(options, function (error, response, body) {
                if (error) {
                    reject(new http_client_response_1.HttpClientResponse(response, error));
                }
                else if (response) {
                    if (expect.includes(response.statusCode)) {
                        accept(new http_client_response_1.HttpClientResponse(response, body));
                    }
                    else {
                        reject(new http_client_response_1.HttpClientResponse(response, error));
                    }
                }
                else {
                    reject(new http_client_response_1.HttpClientResponse(response, 'No response'));
                }
            });
        });
    };
    /**
     * Send a DELETE http request to the server
     * @param path Path to send to (e.g. /users)
     * @param data Data to send as query
     * @param bearer Bearer token to send
     * @param expect Array of status codes to expect
     * @param customOptions Additional request options
     * @return Returns a promise of HttpClientResponse
     */
    HttpClient.prototype.delete = function (path, bearer, expect, customOptions) {
        var _this = this;
        if (bearer === void 0) { bearer = false; }
        if (expect === void 0) { expect = [200, 202, 204]; }
        return new Promise(function (accept, reject) {
            var options = {
                method: 'DELETE',
                url: _this.url + ":" + _this.port + path
            };
            if (bearer) {
                options['auth'] = { bearer: "" + bearer };
            }
            if (_this.headers) {
                options['headers'] = _this.headers;
            }
            if (customOptions) {
                options = Object.assign(options, customOptions);
            }
            _this.request(options, function (error, response, body) {
                if (error) {
                    reject(new http_client_response_1.HttpClientResponse(response, error));
                }
                else if (response) {
                    if (expect.includes(response.statusCode)) {
                        accept(new http_client_response_1.HttpClientResponse(response, body));
                    }
                    else {
                        reject(new http_client_response_1.HttpClientResponse(response, error));
                    }
                }
                else {
                    reject(new http_client_response_1.HttpClientResponse(response, 'No response'));
                }
            });
        });
    };
    return HttpClient;
}());
exports.HttpClient = HttpClient;
//# sourceMappingURL=http-client.js.map