"use strict";
/**
 * # JWT Tokens
 *
 * JWT Authorization headers are the default way to handle authentication.
 *
 * If you use the loginEndpoint and onlySelf/onlyAdmin guards, this will
 * be taken care of for you.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var JWT = require("jsonwebtoken");
var btoa = require('btoa');
var atob = require('atob');
/**
 * JWT Bearer for authentication
 */
var JwtBearer = /** @class */ (function () {
    /**
     * Construct a JWT Bearer token
     * @param key Private key
     */
    function JwtBearer(key) {
        if (key === void 0) { key = 'unset_key'; }
        if (key === 'unset_key' &&
            'JWT_KEY' in process.env &&
            process.env.JWT_KEY) {
            this.key = process.env.JWT_KEY;
        }
        else {
            this.key = key;
        }
    }
    /**
     * Get expiration of the token
     * @return Returns the expiration epoch time
     */
    JwtBearer.prototype.getExpiration = function (isRefresh) {
        if (isRefresh === void 0) { isRefresh = false; }
        return (Date.now() +
            parseInt(isRefresh ? process.env.JWT_REFRESH_TTL : process.env.JWT_TTL, 10) *
                1000);
    };
    /**
     * Sign the token to the User
     * @param user User to sign the JWT for
     * @return Returns the signed, base64-encoded token
     */
    JwtBearer.prototype.sign = function (user, isRefresh, data) {
        if (isRefresh === void 0) { isRefresh = false; }
        if (data === void 0) { data = {}; }
        var payload = Object.assign(data, {
            id: user.id,
            isRefresh: isRefresh
        });
        return btoa(JWT.sign(payload, this.key, {
            expiresIn: parseInt(isRefresh
                ? process.env.JWT_REFRESH_TTL
                : process.env.JWT_TTL, 10)
        }));
    };
    /**
     * Get a token from an HTTP Request
     * @param request Request to fetch the token from
     * @return Returns the token if successful, otherwise false
     */
    JwtBearer.prototype.getToken = function (request) {
        var token = request.header('Authorization');
        if (token && token.includes('Bearer')) {
            return token.replace('Bearer ', '');
        }
        else {
            return false;
        }
    };
    /**
     * Verify the token
     * @param token Token to verify
     * @return Returns the token payload, or false if invalid
     */
    JwtBearer.prototype.dryVerify = function (token) {
        try {
            return JWT.verify(atob(token), this.key);
        }
        catch (ex) {
            return false;
        }
    };
    return JwtBearer;
}());
exports.JwtBearer = JwtBearer;
exports.jwtBearer = new JwtBearer();
//# sourceMappingURL=jwt-bearer.js.map