"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bodyguard_1 = require("../bodyguard");
/**
 * Create a SQL query string for a search
 * @param payloadType Request payload type
 * @param obj Search query object
 * @param objKey SQL query object alias. Default is `obj`
 * @return Returns { queryString, queryParams }
 */
function createSearchQuery(payloadType, obj, objKey) {
    if (objKey === void 0) { objKey = 'obj'; }
    var queryString = '';
    var queryParams = {};
    for (var field in obj) {
        if (field && field === 'where') {
            queryString += '(';
            // Loop through any of keys
            for (var whereKey in obj['where']) {
                // Append key to queryString
                queryString += objKey + "." + whereKey + "=:" + whereKey + " AND ";
                // Append parameter to queryParams
                queryParams[whereKey] = "" + obj['where'][whereKey];
            }
            queryString = queryString.replace(/ AND +$/, '');
            queryString += ') AND ';
        }
        else if (field && field === 'whereAnyOf') {
            queryString += '(';
            // Loop through any of keys
            for (var anyOfKey in obj['whereAnyOf']) {
                // Append key to queryString
                queryString += objKey + "." + anyOfKey + "=:" + anyOfKey + " OR ";
                // Append parameter to queryParams
                queryParams[anyOfKey] = "" + obj['whereAnyOf'][anyOfKey];
            }
            queryString = queryString.replace(/ OR +$/, '');
            queryString += ') AND ';
        }
        else if (field && field === 'search') {
            var ptype = new payloadType();
            var searchableFields = bodyguard_1.getSearchableFields(ptype);
            var searchableRelations = bodyguard_1.getSearchableRelations(ptype);
            queryString += '(';
            if (typeof obj[field] === 'string') {
                searchableFields.forEach(function (key) {
                    // Append searchable key to queryString
                    queryString += "LOWER(" + objKey + "." + key + ") LIKE :search OR ";
                    // Append parameter to queryParams (with wildcards)
                    var value = obj['search']
                        .replace(/[\s]+/, '%')
                        .toLowerCase();
                    queryParams['search'] = "%" + value + "%";
                });
                searchableRelations.forEach(function (key) {
                    // Append searchable key to queryString
                    queryString += "LOWER(" + key + ") LIKE :search OR ";
                    // Append parameter to queryParams (with wildcards)
                    var value = obj['search']
                        .replace(/[\s]+/, '%')
                        .toLowerCase();
                    queryParams['search'] = "%" + value + "%";
                });
            }
            else if (typeof obj[field] === 'object') {
                for (var key in obj[field]) {
                    // Append searchable key to queryString
                    queryString += "LOWER(" + objKey + "." + key + ") LIKE :search_" + key + " OR ";
                    // Append parameter to queryParams (with wildcards)
                    var value = obj[field][key]
                        .replace(/[\s]+/, '%')
                        .toLowerCase();
                    queryParams['search_' + key] = "%" + value + "%";
                }
            }
            queryString = queryString.replace(/ OR +$/, '');
            queryString += ') AND ';
        }
        else if (field && field === 'between') {
            for (var betweenKey in obj['between']) {
                var range = obj['between'][betweenKey];
                // Range should be an array of two
                if ('length' in range && range.length === 2) {
                    // Range should be int
                    range.map(function (val) {
                        return parseInt(val, 10);
                    });
                    // Append key to queryString
                    queryString +=
                        "(" + objKey + "." + betweenKey + " BETWEEN " +
                            range[0] +
                            ' AND ' +
                            range[1] +
                            ') AND ';
                }
            }
        }
        else if (field && field === 'lessThan') {
            for (var lessThanKey in obj['lessThan']) {
                // Append key to queryString
                queryString += objKey + "." + lessThanKey + " < :" + lessThanKey + " AND ";
                // Append parameter to queryParams
                queryParams[lessThanKey] = "" + obj['lessThan'][lessThanKey];
            }
        }
        else if (field && field === 'greaterThan') {
            for (var greaterThanKey in obj['greaterThan']) {
                // Append key to queryString
                queryString += objKey + "." + greaterThanKey + " > :" + greaterThanKey + " AND ";
                // Append parameter to queryParams
                queryParams[greaterThanKey] = "" + obj['greaterThan'][greaterThanKey];
            }
        }
        else if (field && field === 'lessThanOrEqual') {
            for (var lessThanOrEqualKey in obj['lessThanOrEqual']) {
                // Append key to queryString
                queryString +=
                    objKey + "." + lessThanOrEqualKey + " <= " +
                        (":" + lessThanOrEqualKey + " AND ");
                // Append parameter to queryParams
                queryParams[lessThanOrEqualKey] = "" + obj['lessThanOrEqual'][lessThanOrEqualKey];
            }
        }
        else if (field && field === 'greaterThanOrEqual') {
            for (var greaterThanOrEqualKey in obj['greaterThanOrEqual']) {
                // Append key to queryString
                queryString +=
                    objKey + "." + greaterThanOrEqualKey + " >= " +
                        (":" + greaterThanOrEqualKey + " AND ");
                // Append parameter to queryParams
                queryParams[greaterThanOrEqualKey] = "" + obj['greaterThanOrEqual'][greaterThanOrEqualKey];
            }
        }
        else if (field && field === 'not') {
            for (var notKey in obj['not']) {
                // Append key to queryString
                queryString += objKey + "." + notKey + "!=:" + notKey + " AND ";
                // Append parameter to queryParams
                queryParams[notKey] = "" + obj['not'][notKey];
            }
        }
    }
    queryString = queryString.replace(/ AND +$/, '');
    return { queryString: queryString, queryParams: queryParams };
}
exports.createSearchQuery = createSearchQuery;
//# sourceMappingURL=create-search-query.js.map