"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../utils");
var query_types_1 = require("./query-types");
var bodyguard_1 = require("../bodyguard");
var class_validator_1 = require("class-validator");
var get_validation_constraints_1 = require("./get-validation-constraints");
/**
 * Validate a GET query type
 * @param type Query type name
 * @param request Express request
 * @param response Express response
 * @return Returns if the currrent query type is valid
 */
function queryFieldValidator(type, request, response) {
    var readableKeys = bodyguard_1.getReadableFields(new request.payloadType(), request.user);
    var readableRelations = bodyguard_1.getReadableRelations(new request.payloadType(), request.user);
    if (type in request.query) {
        if (request.query[type] instanceof Array) {
            for (var i = 0; i < request.query[type].length; i++) {
                var key = request.query[type][i];
                if (key === undefined) {
                    request.query[type].splice(i, 1);
                    continue;
                }
                if (!utils_1.isKeyInModel(key, request.payload, response)) {
                    response.validationResponder('Member "' + key + '" does not exist in model');
                    return false;
                }
                if (!readableKeys.includes(key) &&
                    !readableRelations.includes(key && key.indexOf('.') ? key.split('.')[0] : key)) {
                    response.forbiddenResponder('Cannot "' + type + '" by member "' + key + '"');
                    return false;
                }
            }
        }
        else if (request.query[type] instanceof Object) {
            var validators = get_validation_constraints_1.getValidationConstraints(request.payloadType);
            for (var key in request.query[type]) {
                var value = request.query[type][key];
                if (value === undefined) {
                    delete request.query[type][key];
                    continue;
                }
                if (!utils_1.isKeyInModel(key, request.payload, response)) {
                    response.validationResponder('Member "' + key + '" does not exist in model');
                    return false;
                }
                if (!readableKeys.includes(key) &&
                    !readableRelations.includes(key && key.indexOf('.') ? key.split('.')[0] : key)) {
                    response.forbiddenResponder('Cannot "' + type + '" by member "' + key + '"');
                    return false;
                }
                // Cast to int if need be
                if (key in validators &&
                    'isInt' in validators[key] &&
                    validators[key]['isInt'] === true) {
                    if (Array.isArray(value)) {
                        for (var i = 0; i < value.length; i++) {
                            var asInt = parseInt(value[i], 10);
                            if (!isNaN(asInt)) {
                                value[i] = asInt;
                            }
                        }
                    }
                    else {
                        var asInt = parseInt(value, 10);
                        if (!isNaN(asInt)) {
                            request.query[type][key] = asInt;
                        }
                    }
                }
            }
            if (type === 'where' ||
                type === 'whereAnyOf' ||
                type === 'not' ||
                type === 'lessThan' ||
                type === 'lessThanOrEqual' ||
                type === 'greaterThan' ||
                type === 'greaterThanOrEqual') {
                var testObject = Object.assign(new request.payloadType(), request.query[type]);
                var validationErrors = class_validator_1.validateSync(testObject, {
                    skipMissingProperties: true
                });
                if (validationErrors && validationErrors.length) {
                    response.validationResponder(validationErrors);
                    return false;
                }
            }
        }
        else if (type === 'id') {
            var validator_1 = get_validation_constraints_1.getValidationConstraints(request.payloadType, 'id');
            if (validator_1 && 'isInt' in validator_1 && validator_1.isInt === true) {
                var asInt = parseInt(request.query.id, 10);
                if (!isNaN(asInt)) {
                    request.query.id = asInt;
                }
            }
            var testObject = Object.assign(new request.payloadType(), {
                id: request.query[type]
            });
            var validationErrors = class_validator_1.validateSync(testObject, {
                skipMissingProperties: true
            });
            if (validationErrors && validationErrors.length) {
                response.validationResponder(validationErrors);
                return false;
            }
        }
    }
    return true;
}
/**
 * Validate a GET query
 * @param request Express::Request Express request
 * @param response Express::Response Express response
 * @return Returns if the get query is valid
 */
function queryValidator(request, response) {
    for (var type in request.query) {
        // Is this defined?
        if (request.query[type] === undefined) {
            delete request.query[type];
            continue;
        }
        // Is query type registered?
        if (!query_types_1.queryTypeKeys.includes(type)) {
            if (response) {
                response.validationResponder('Query type "' + type + '" is not a valid query type.');
            }
            return false;
        }
        // Is query type proper (array, object, etc)?
        var queryTypeConstructor = typeof request.query[type];
        if (queryTypeConstructor === 'object' &&
            'length' in request.query[type]) {
            queryTypeConstructor = 'array';
        }
        if (!query_types_1.queryTypes[type].includes(queryTypeConstructor)) {
            if (response) {
                response.validationResponder('Query type "' +
                    type +
                    '" should be one of: [' +
                    query_types_1.queryTypes[type].join() +
                    ']. It is: ' +
                    queryTypeConstructor +
                    '. Value: ' +
                    request.query[type]);
            }
            return false;
        }
        // Is current query valid?
        if (type !== 'additionalParameters' &&
            !queryFieldValidator(type, request, response)) {
            return false;
        }
    }
    return true;
}
exports.queryValidator = queryValidator;
//# sourceMappingURL=query-validator.js.map